/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.biz.modular.bizaccessrecord.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.v3.oas.annotations.tags.Tag;
import io.swagger.v3.oas.annotations.Operation;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;
import vip.xiaonuo.common.annotation.CommonLog;
import vip.xiaonuo.common.pojo.CommonResult;
import vip.xiaonuo.biz.modular.bizaccessrecord.entity.BizAccessRecord;
import vip.xiaonuo.biz.modular.bizaccessrecord.param.BizAccessRecordAddParam;
import vip.xiaonuo.biz.modular.bizaccessrecord.param.BizAccessRecordEditParam;
import vip.xiaonuo.biz.modular.bizaccessrecord.param.BizAccessRecordIdParam;
import vip.xiaonuo.biz.modular.bizaccessrecord.param.BizAccessRecordPageParam;
import vip.xiaonuo.biz.modular.bizaccessrecord.service.BizAccessRecordService;

import jakarta.annotation.Resource;
import jakarta.validation.Valid;
import jakarta.validation.constraints.NotEmpty;
import java.util.List;

/**
 * 门禁记录表控制器
 *
 * @author fanzherong
 * @date  2025/03/25 16:48
 */
@Tag(name = "门禁记录表控制器")
@RestController
@Validated
public class BizAccessRecordController {

    @Resource
    private BizAccessRecordService bizAccessRecordService;

    /**
     * 获取门禁记录表分页
     *
     * @author fanzherong
     * @date  2025/03/25 16:48
     */
    @Operation(summary = "获取门禁记录表分页")
    @SaCheckPermission("/biz/bizaccessrecord/page")
    @GetMapping("/biz/bizaccessrecord/page")
    public CommonResult<Page<BizAccessRecord>> page(BizAccessRecordPageParam bizAccessRecordPageParam) {
        return CommonResult.data(bizAccessRecordService.page(bizAccessRecordPageParam));
    }

    /**
     * 添加门禁记录表
     *
     * @author fanzherong
     * @date  2025/03/25 16:48
     */
    @Operation(summary = "添加门禁记录表")
    @CommonLog("添加门禁记录表")
    @PostMapping("/biz/bizaccessrecord/add")
    public CommonResult<String> add(@RequestBody @Valid BizAccessRecordAddParam bizAccessRecordAddParam) {
        bizAccessRecordService.add(bizAccessRecordAddParam);
        return CommonResult.ok();
    }

    /**
     * 编辑门禁记录表
     *
     * @author fanzherong
     * @date  2025/03/25 16:48
     */
    @Operation(summary = "编辑门禁记录表")
    @CommonLog("编辑门禁记录表")
    @PostMapping("/biz/bizaccessrecord/edit")
    public CommonResult<String> edit(@RequestBody @Valid BizAccessRecordEditParam bizAccessRecordEditParam) {
        bizAccessRecordService.edit(bizAccessRecordEditParam);
        return CommonResult.ok();
    }

    /**
     * 删除门禁记录表
     *
     * @author fanzherong
     * @date  2025/03/25 16:48
     */
    @Operation(summary = "删除门禁记录表")
    @CommonLog("删除门禁记录表")
    @PostMapping("/biz/bizaccessrecord/delete")
    public CommonResult<String> delete(@RequestBody @Valid @NotEmpty(message = "集合不能为空")
                                                   List<BizAccessRecordIdParam> bizAccessRecordIdParamList) {
        bizAccessRecordService.delete(bizAccessRecordIdParamList);
        return CommonResult.ok();
    }

    /**
     * 获取门禁记录表详情
     *
     * @author fanzherong
     * @date  2025/03/25 16:48
     */
    @Operation(summary = "获取门禁记录表详情")
    @GetMapping("/biz/bizaccessrecord/detail")
    public CommonResult<BizAccessRecord> detail(@Valid BizAccessRecordIdParam bizAccessRecordIdParam) {
        return CommonResult.data(bizAccessRecordService.detail(bizAccessRecordIdParam));
    }
}
