/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.biz.modular.bizorder.param;

import io.swagger.v3.oas.annotations.media.Schema;
import lombok.Getter;
import lombok.Setter;

import jakarta.validation.constraints.NotBlank;
import jakarta.validation.constraints.NotNull;
import java.math.BigDecimal;
import java.util.Date;

/**
 * 订单管理添加参数
 *
 * @author fanzherong
 * @date  2025/03/21 17:16
 **/
@Getter
@Setter
public class BizOrderAddParam {

    /** 订单编号 */
    @Schema(description = "订单编号")
    private String orderNumber;

    /** 订单名称 */
    @Schema(description = "订单名称", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotBlank(message = "orderName不能为空")
    private String orderName;

    /** 客户id */
    @Schema(description = "客户id", requiredMode = Schema.RequiredMode.REQUIRED)
    private String customerId;

    /** 货品id */
    @Schema(description = "货品id", requiredMode = Schema.RequiredMode.REQUIRED)
    private String goodId;

    /** 订单类型（1:采购   2：销售） */
    @Schema(description = "订单类型（1:采购   2：销售）")
    private String orderType;

    /** 订单来源（1：oa推送   2：手动新增） */
    @Schema(description = "订单来源（1：oa推送   2：手动新增）")
    private String orderSource;

    /** 订单状态（1：正常  2：终止） */
    @Schema(description = "订单状态（1：正常  2：终止）")
    private String orderStatus;

    /** 订单重量 */
    @Schema(description = "订单重量", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotNull(message = "orderWeight不能为空")
    private BigDecimal orderWeight;

    /** 过磅重量 */
    @Schema(description = "过磅重量")
    private BigDecimal netWeight;

}
