/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.biz.modular.goodsConf.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.v3.oas.annotations.tags.Tag;
import io.swagger.v3.oas.annotations.Operation;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;
import vip.xiaonuo.common.annotation.CommonLog;
import vip.xiaonuo.common.pojo.CommonResult;
import vip.xiaonuo.biz.modular.goodsConf.entity.BizGoodsConf;
import vip.xiaonuo.biz.modular.goodsConf.param.BizGoodsConfAddParam;
import vip.xiaonuo.biz.modular.goodsConf.param.BizGoodsConfEditParam;
import vip.xiaonuo.biz.modular.goodsConf.param.BizGoodsConfIdParam;
import vip.xiaonuo.biz.modular.goodsConf.param.BizGoodsConfPageParam;
import vip.xiaonuo.biz.modular.goodsConf.service.BizGoodsConfService;

import jakarta.annotation.Resource;
import jakarta.validation.Valid;
import jakarta.validation.constraints.NotEmpty;
import java.util.List;

/**
 * 提货时间配置表控制器
 *
 * @author sandy
 * @date  2025/03/27 10:54
 */
@Tag(name = "提货时间配置表控制器")
@RestController
@Validated
public class BizGoodsConfController {

    @Resource
    private BizGoodsConfService goodsConfService;

    /**
     * 获取提货时间配置表分页
     *
     * @author sandy
     * @date  2025/03/27 10:54
     */
    @Operation(summary = "获取提货时间配置表分页")
    @GetMapping("/biz/goodsConf/page")
    public CommonResult<Page<BizGoodsConf>> page(BizGoodsConfPageParam goodsConfPageParam) {
        return CommonResult.data(goodsConfService.page(goodsConfPageParam));
    }

    /**
     * 添加提货时间配置表
     *
     * @author sandy
     * @date  2025/03/27 10:54
     */
    @Operation(summary = "添加提货时间配置表")
    @CommonLog("添加提货时间配置表")
    @SaCheckPermission("/biz/goodsConf/add")
    @PostMapping("/biz/goodsConf/add")
    public CommonResult<String> add(@RequestBody @Valid BizGoodsConfAddParam goodsConfAddParam) {
        goodsConfService.add(goodsConfAddParam);
        return CommonResult.ok();
    }

    /**
     * 编辑提货时间配置表
     *
     * @author sandy
     * @date  2025/03/27 10:54
     */
    @Operation(summary = "编辑提货时间配置表")
    @CommonLog("编辑提货时间配置表")
    @SaCheckPermission("/biz/goodsConf/edit")
    @PostMapping("/biz/goodsConf/edit")
    public CommonResult<String> edit(@RequestBody @Valid BizGoodsConfEditParam goodsConfEditParam) {
        goodsConfService.edit(goodsConfEditParam);
        return CommonResult.ok();
    }

    /**
     * 删除提货时间配置表
     *
     * @author sandy
     * @date  2025/03/27 10:54
     */
    @Operation(summary = "删除提货时间配置表")
    @CommonLog("删除提货时间配置表")
    @SaCheckPermission("/biz/goodsConf/delete")
    @PostMapping("/biz/goodsConf/delete")
    public CommonResult<String> delete(@RequestBody @Valid @NotEmpty(message = "集合不能为空")
                                                   List<BizGoodsConfIdParam> goodsConfIdParamList) {
        goodsConfService.delete(goodsConfIdParamList);
        return CommonResult.ok();
    }

    /**
     * 获取提货时间配置表详情
     *
     * @author sandy
     * @date  2025/03/27 10:54
     */
    @Operation(summary = "获取提货时间配置表详情")
    @GetMapping("/biz/goodsConf/detail")
    public CommonResult<BizGoodsConf> detail(@Valid BizGoodsConfIdParam goodsConfIdParam) {
        return CommonResult.data(goodsConfService.detail(goodsConfIdParam));
    }

    /**
     * 大于当前时间的获取货品列表
     * 参数：goodsCode 货品编码  完全匹配
     * 参数：goodsName 货品名称  完全匹配
     * 参数：needWeight 需要重量  大于等于
     * @author sandy
     * @date  2025/03/27 14:23
     */
    @Operation(summary = "获取货品管理详情")
    @GetMapping("/biz/goodsConf/getList")
    public CommonResult<List<BizGoodsConf>> getList(BizGoodsConfPageParam goodsConfPageParam) {
        return CommonResult.data(goodsConfService.list(goodsConfPageParam));
    }
}
