/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.biz.modular.supplier.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.v3.oas.annotations.tags.Tag;
import io.swagger.v3.oas.annotations.Operation;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;
import vip.xiaonuo.common.annotation.CommonLog;
import vip.xiaonuo.common.pojo.CommonResult;
import vip.xiaonuo.biz.modular.supplier.entity.BizSupplier;
import vip.xiaonuo.biz.modular.supplier.param.BizSupplierAddParam;
import vip.xiaonuo.biz.modular.supplier.param.BizSupplierEditParam;
import vip.xiaonuo.biz.modular.supplier.param.BizSupplierIdParam;
import vip.xiaonuo.biz.modular.supplier.param.BizSupplierPageParam;
import vip.xiaonuo.biz.modular.supplier.service.BizSupplierService;

import jakarta.annotation.Resource;
import jakarta.validation.Valid;
import jakarta.validation.constraints.NotEmpty;
import java.util.List;

/**
 * 供货商控制器
 *
 * @author fanzherong
 * @date  2025/03/21 10:38
 */
@Tag(name = "供货商控制器")
@RestController
@Validated
public class BizSupplierController {

    @Resource
    private BizSupplierService bizSupplierService;

    /**
     * 获取供货商分页
     *
     * @author fanzherong
     * @date  2025/03/21 10:38
     */
    @Operation(summary = "获取供货商分页")
    @GetMapping("/biz/supplier/page")
    public CommonResult<Page<BizSupplier>> page(BizSupplierPageParam bizSupplierPageParam) {
        return CommonResult.data(bizSupplierService.page(bizSupplierPageParam));
    }

    /**
     * 添加供货商
     *
     * @author fanzherong
     * @date  2025/03/21 10:38
     */
    @Operation(summary = "添加供货商")
    @CommonLog("添加供货商")
    @SaCheckPermission("/biz/supplier/add")
    @PostMapping("/biz/supplier/add")
    public CommonResult<String> add(@RequestBody @Valid BizSupplierAddParam bizSupplierAddParam) {
        bizSupplierService.add(bizSupplierAddParam);
        return CommonResult.ok();
    }

    /**
     * 编辑供货商
     *
     * @author fanzherong
     * @date  2025/03/21 10:38
     */
    @Operation(summary = "编辑供货商")
    @CommonLog("编辑供货商")
    @SaCheckPermission("/biz/supplier/edit")
    @PostMapping("/biz/supplier/edit")
    public CommonResult<String> edit(@RequestBody @Valid BizSupplierEditParam bizSupplierEditParam) {
        bizSupplierService.edit(bizSupplierEditParam);
        return CommonResult.ok();
    }

    /**
     * 删除供货商
     *
     * @author fanzherong
     * @date  2025/03/21 10:38
     */
    @Operation(summary = "删除供货商")
    @CommonLog("删除供货商")
    @SaCheckPermission("/biz/supplier/delete")
    @PostMapping("/biz/supplier/delete")
    public CommonResult<String> delete(@RequestBody @Valid @NotEmpty(message = "集合不能为空")
                                                   List<BizSupplierIdParam> bizSupplierIdParamList) {
        bizSupplierService.delete(bizSupplierIdParamList);
        return CommonResult.ok();
    }

    /**
     * 获取供货商详情
     *
     * @author fanzherong
     * @date  2025/03/21 10:38
     */
    @Operation(summary = "获取供货商详情")
    @GetMapping("/biz/supplier/detail")
    public CommonResult<BizSupplier> detail(@Valid BizSupplierIdParam bizSupplierIdParam) {
        return CommonResult.data(bizSupplierService.detail(bizSupplierIdParam));
    }
}
